import 'package:cresce_client/http_requests/formaters/decoders.dart';
import 'package:cresce_client/http_requests/http_client.dart';
import 'package:http/http.dart';
import 'package:http/testing.dart';
import 'package:test/test.dart';

class HttpClientMock extends MockClient {
  bool wasClosed;

  HttpClientMock(fn) : super(fn);

  @override
  void close() {
    super.close();
    wasClosed = true;
  }
}

class HttpGetMocked extends HttpGet {
  HttpClientMock _client;

  HttpGetMocked(String authority, [MockClientHandler fn])
      : _client = HttpClientMock(fn),
        super(authority, JsonFormatter());

  @override
  makeClient() => _client;
}

main() {
  test('getting to url concatenates with authority', () async {
    var requestUrl;
    var httpGet = HttpGetMocked('myAuthority/', (request) {
      requestUrl = request.url.toString();
      return Future.value(Response("", 200));
    });

    await httpGet.get('test');

    expect(requestUrl, 'myAuthority/test');
  });

  test('getting closes client', () async {
    var httpGet = HttpGetMocked('myAuthority/', (request) {
      return Future.value(Response('', 200));
    });

    await httpGet.get('test');

    expect(httpGet._client.wasClosed, isTrue);
  });

  test('getting sets content type to headers', () async {
    var contentType;
    var httpGet = HttpGetMocked('myAuthority', (request) {
      contentType = request.headers['content-type'];
      return Future.value(Response('', 200));
    });

    await httpGet.get('test');

    expect(contentType, 'application/json; charset=utf-8');
  });

  [200, 201].forEach((statusCode) {
    test('getting returns wasSuccess when status code is $statusCode',
        () async {
      var httpGet = HttpGetMocked('myAuthority', (request) {
        return Future.value(Response('', statusCode));
      });

      var result = await httpGet.get('test');

      expect(result.wasSuccess(), isTrue);
    });
  });

  test('getting returns result that can be deserialized', () async {
    var httpGet = HttpGetMocked('myAuthority', (request) {
      return Future.value(Response('{}', 200));
    });

    var result = await httpGet.get('test');

    expect(result.deserialize<TestModel>(TestModel()), isNotNull);
  });
}

class TestModel implements Serializable, Deserialize {
  @override
  String serialize(Encoder encoder) {
    return encoder.encode({
      'test': 'data',
    });
  }

  @override
  Object deserialize(Object data) {
    return this;
  }
}
