import 'package:cresce_client/http_requests/formaters/decoders.dart';
import 'package:cresce_client/http_requests/http_client.dart';
import 'package:http/http.dart';
import 'package:http/testing.dart';
import 'package:test/test.dart';

class HttpClientMock extends MockClient {
  bool wasClosed;

  HttpClientMock(fn) : super(fn);

  @override
  void close() {
    super.close();
    wasClosed = true;
  }
}

class HttpPostMocked extends HttpPost {
  HttpClientMock _client;

  HttpPostMocked(String authority, [MockClientHandler fn])
      : _client = HttpClientMock(fn),
        super(authority, JsonFormatter());

  @override
  makeClient() => _client;
}

main() {
  test('posting to url concatenates with authority', () async {
    var requestUrl;
    var httpPost = HttpPostMocked('myAuthority/', (request) {
      requestUrl = request.url.toString();
      return Future.value(Response("", 200));
    });

    await httpPost.post('test');

    expect(requestUrl, 'myAuthority/test');
  });

  test('posting closes client', () async {
    var httpPost = HttpPostMocked('myAuthority/', (request) {
      return Future.value(Response('', 200));
    });

    await httpPost.post('test');

    expect(httpPost._client.wasClosed, isTrue);
  });

  test('posting sets content type to headers', () async {
    var contentType;
    var httpPost = HttpPostMocked('myAuthority', (request) {
      contentType = request.headers['content-type'];
      return Future.value(Response('', 200));
    });

    await httpPost.post('test');

    expect(contentType, 'application/json; charset=utf-8');
  });

  test('posting sends formatted body', () async {
    var body;
    var httpPost = HttpPostMocked('myAuthority', (request) {
      body = request.body;
      return Future.value(Response('', 200));
    });

    await httpPost.post('', TestModel());

    expect(body, '{"test":"data"}');
  });

  [200, 201].forEach((statusCode) {
    test('posting returns wasSuccess when status code is $statusCode',
        () async {
      var httpPost = HttpPostMocked('myAuthority', (request) {
        return Future.value(Response('', statusCode));
      });

      var result = await httpPost.post('test');

      expect(result.wasSuccess(), isTrue);
    });
  });

  test('posting returns result that can be deserialized', () async {
    var httpPost = HttpPostMocked('myAuthority', (request) {
      return Future.value(Response('{}', 200));
    });

    var result = await httpPost.post('test');

    expect(result.deserialize<TestModel>(TestModel()), isNotNull);
  });
}

class TestModel implements Serializable, Deserialize {
  @override
  String serialize(Encoder encoder) {
    return encoder.encode({
      'test': 'data',
    });
  }

  @override
  Object deserialize(Object data) {
    return this;
  }
}
