import 'package:cresce_client/cresce_client.dart';
import 'package:cresce_client/http_requests/formaters/decoders.dart';
import 'package:cresce_client/http_requests/http_client.dart';
import 'package:equatable/equatable.dart';
import 'package:mockito/mockito.dart';
import 'package:test/test.dart';

import 'monitor.dart';

class HttpPostMock extends Mock implements HttpPost {}

void main() {
  EquatableConfig.stringify = true;
  Monitor monitor;
  LoginResultDto loginResult;
  bool failed = false;

  setUp(() {
    monitor = Monitor();
  });

  void onSuccess(LoginResultDto result) {
    loginResult = result;
    monitor.signal();
  }

  void onFailure() {
    failed = true;
    monitor.signal();
  }

  test('on successful login calls given callback function', () async {
    final loginServices = makeServices();

    loginServices.login(
      CredentialsDto(user: 'myUser', password: 'myPass'),
      successCallback: onSuccess,
    );
    await monitor.wait();

    expect(loginResult, isNotNull);
    expect(loginResult.organizationUrl, 'someOrganizationUrl');
    expect(loginResult.token, 'myAuthToken');
  });

  test('on successful login without callback doesnt fail', () async {
    final loginServices = makeServices();

    loginServices.login(CredentialsDto(user: 'myUser', password: 'myPass'));

    await monitor.wait();
  });

  test('on failure login calls given callback function', () async {
    final loginServices = makeServices();

    loginServices.login(
      CredentialsDto(user: 'myUser1', password: 'myPass'),
      failureCallback: onFailure,
    );

    await monitor.wait();

    expect(failed, isTrue);
  });

  test('on failure login without callback doesnt fail', () async {
    final loginServices = makeServices();

    loginServices.login(
      CredentialsDto(user: 'myUser1', password: 'myPass'),
    );

    await monitor.wait();
  });
}

LoginServices makeServices() => LoginServices(makeHttpPost());

HttpPostMock makeHttpPost() {
  var http = HttpPostMock();
  var formatter = JsonFormatter();

  when(http.post(
    'api/v1/authentication/',
    CredentialsDto(user: 'myUser', password: 'myPass'),
  )).thenAnswer(
    (_) {
      return Future.value(HttpResult(
        formatter,
        statusCode: 200,
        body: LoginResultDto(
          organizationUrl: 'someOrganizationUrl',
          token: 'myAuthToken',
        ).serialize(formatter),
      ));
    },
  );

  when(http.post(
    'api/v1/authentication/',
    CredentialsDto(user: 'myUser1', password: 'myPass'),
  )).thenAnswer(
    (_) {
      return Future.value(HttpResult(
        formatter,
        statusCode: 500,
      ));
    },
  );

  return http;
}
